 
{
/**************************************************************************
 * Copyright [c] <2012-2013>  - "TACHYON Forth" for the Parallax Propeller
 *                              Peter Jakacki, Brisbane, Australia
 *
 *           [c] <2013>       - Tachyon Forth source code for PULSOX-T
 *                              Brian Riley, Underhill Ctr., VT, USA
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 *     The above copyright notice and this permission notice shall be 
 *     included in all copies or substantial portions of the Software.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
 *   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 *   BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN 
 *   ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
 *   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 *   SOFTWARE.
 *
 **************************************************************************/
}

TACHYON

{
						DSRTC.fth

	UPDATED 20130817.1900 - BBR - completed integrating PBJ's words
								  added more print displayformats
								
 	UPDATED 20130815.1800 - BBR - adapted for additional use with DS3231M
								  borrowed some routines from PBJ's 
								  MCP79410 code

	CREATED 20121223.0500 - BB Riley, Underhill Center, VT  
                        brianbr@wulfden.org

	TESTED  Tachyon Forth V21 20130811 Extend 20130814

	TESTED  Tachyon Forth V21 20121214.1800 Extend 20121214.1900

	DS1307/DS3231 I2C ADDRESS IS 7 BITS = $68 left justified in the byte
    %110_1000 ==> %1101_0000 becomes $D0
}

: DSRTC.fth 	." DS1307/3231 RTC ToolKit - 20130817.1900" ;

IFNDEF rtcbuf
	BYTE	rtcbuf	8 ALLOT
}

1	CONSTANT	DS3231	\ 	Comment out one of these two chips, thus selecting the
\ 1	CONSTANT	DS1307	\	other chip. This will then link the correct speed I2C
						\	i/o primitives to RTC@, RTC!,,, and RTC!? ... slow for
						\	the DS1307 and fast for DS3231

$D0	CONSTANT	@rtc	\	both chips have the same I2C address

 0	CONSTANT	DSWRITE
 1	CONSTANT	DSREAD

 0	CONSTANT	DSSEC	\  seconds 0-59
 1	CONSTANT	DSMIN	\  minutes 0-59
 2	CONSTANT	DSHR	\  hours 0-23
 3	CONSTANT	DSDOW	\  day of week Sun=1 ... Sat=7
 4	CONSTANT	DSDAY	\  day of month
 5	CONSTANT	DSMON	\  month Jan=1 ... Dec=12
 6	CONSTANT	DSYR	\  year 0-99
 7	CONSTANT	DSCTL	\  control register

IFNDEF	DS3231
				\  although Tachyon Forth recognizes the DS1307 as  a 'fast' device, IT IS
				\  NOT!!! It needs to be run at 125K and NOT at 400K. Hence Peter's (S)low I2C 
				\  i/o is selected
				
:	RTC!	( byte -- )				SI2C! ;	
:	RTC!?	( byte -- flag )		SI2C!? ;
:	RTC@	( flag -- byte flag )	SI2C@  ;
}

IFNDEF DS1307
				\ DS3231 can keep up at 400K
				
:	RTC!	( byte -- )				I2C! ;	
:	RTC!?	( byte -- flag )		I2C!? ;
:	RTC@	( flag -- byte flag )	I2C@  ;
}

IFNDEF _EMITS 
:	_EMITS	( addr cnt -- )		ADO I C@ EMIT LOOP  ;
}
           
:	BCD>INT	( bcd -- int )	DUP $0F AND SWAP 4 SHR $0F AND #10 * + ;
:	INT>BCD	( int -- bcd )	DUP #10 / $10 * SWAP #10 MOD + ;

:	DAY$	( index -- addr )   3 * " ---SunMonTueWedThuFriSat" + ;
:	MONTH$	( index -- addr )   3 * " ---JanFebMarAprMayJunJulAugSepOctNovDec" + ;

:	RTCMODE	( r/w -- flag )		\  sets slave read/write mode		
			I2CSTART @rtc OR RTC!? DROP ;
			
:	RTCREG	( reg -- flag )		\  sets slave to register #
			DSWRITE RTCMODE RTC!? DROP ;
				
:	RTCREG!	( reg val -- flag )	 \  writes <val> to slave register #
			SWAP RTCREG RTC!? I2CSTOP ;

:	RTCVAL@	( index -- val )	rtcbuf + C@  ;	\  get BCD value for  hub RAM

:	RDRTC	( -- )				\  reads date/time registers from slave to memory
			0 RTCREG	DSREAD RTCMODE
			rtcbuf 8 ADO 
				0 RTC@ I C!		\  we keep the time/date numbers in BCD because its easer
			LOOP				\  to  print using .BYTE - we convert to INT only for
			1 RTC@	I2CSTOP		\  indexing into the strings for month and day of week
			;				

:	WRRTC	( -- )			\  reads date/time registers from memory to slave
			I2CSTART @rtc RTC! 0 RTC!
			rtcbuf 8 ADO I C@ RTC! LOOP
			I2CSTOP
			;			
			
pub TIME@ ( -- $hhmmss ) \ read time in bcd format
        RDRTC rtcbuf @ $FFFF7F AND ;

pub DATE@ ( -- $yymmdd ) \ read international date in bcd format
        RDRTC rtcbuf DSDAY + @ $FF1F3F AND ;

\ ---------------- Time / Date / DayofWeek Setting -----------------

:	DAY!	( dayofweek -- ) DSDOW rtcbuf + C! WRRTC  ;
							\ single digit 1-7  \ Sun = 1 ... Sat = 7

							\ TIME and DATE are entered in BCD format
							
:	TIME!	( $hh.mm.ss -- )	$2000.0080 OR rtcbuf ! WRRTC ;	\ Ex. 4115705 TIME!

:	DATE!	( $yy.mm.dd -- )	rtcbuf 4 + ! WRRTC ;			\ Ex. $130817 DATE!

\ ------------------- Date / Time Print Words  ----------------------
			
:	.REGVAL	( index -- )		RTCVAL@ .BYTE CR ;
:	.MONTH	( index -- )		MONTH$ 3 _EMITS  ;
:	.DAY	( index -- )		DAY$ 3 _EMITS  ;

:	.TIME	( -- )		\  print time string hh:mm:ss
			TIME@ L>W .BYTE ." :" W>B .BYTE ." :" .BYTE ;

			\ Print date in international format YYYY/MM/DD (otherwise 1/12/2013 
			\ could be 1st of December or 12th of January)
			
pub .DATEI  ( -- )		DATE@ L>W ." 20" .BYTE ." /" W>B .BYTE ." /" .BYTE ;                
			
:	.DATE	( -- )		\  print date string  mm/dd/20yy
			RDRTC
			DSMON	RTCVAL@ .BYTE "/" EMIT
			DSDAY	RTCVAL@ .BYTE ." /20"
			DSYR	RTCVAL@ .BYTE
			;

pub	.DT		( -- )	.DATE  SPACE .TIME ;        
:	.DTI	( -- )	.DATEI SPACE .TIME ;        


\  -------------- Demo Display -------------------------			
:	.DATETIME	( -- )
			CR ." ----------" CR
			RDRTC
			DSDOW	RTCVAL@		BCD>INT	.DAY ." , "	\  DoW
			DSDAY	RTCVAL@		.BYTE  SPACE		\  Day
			DSMON	RTCVAL@ 	BCD>INT .MONTH 		\  Month	
			DSYR	RTCVAL@  ."  20" .BYTE 			\  Year
:	.TIMELOOP		( -- )
			CR CR
			BEGIN	
				4 SPACES .TIME 4 SPACES #920 ms $0D EMIT 
			KEY? UNTIL
			DROP
			;

\  ------------ Debugging Tools --------------------
			
:	RTCREG@	( index -- )		\  reads and display contents of slave register
			RTCREG	DSREAD RTCMODE
			0 RTC@ ." Value = " .BYTE CR
			1 RTC@	I2CSTOP
			;

:	DREGS	( -- ) RDRTC 				\ dumps RTC regs to RTC buffer
:	DRTC	( -- ) rtcbuf $20 DUMP ;	\ dumps RTC buffer to console

:	READRTC		( -- )
			RDRTC
			CR ." ----------" CR
			." SEC = "	DSSEC .REGVAL
			." MIN = "  DSMIN .REGVAL
			." HRS = "  DSHR .REGVAL
			." DOW = "  DSDOW .REGVAL
			." DAY = "  DSDAY .REGVAL
			." MON = "  DSMON .REGVAL
			." YR  = "  DSYR .REGVAL
			." CTL = "  DSCTL .REGVAL
			;
			
			
			
END
